@@ -218,6 +218,10 @@ class ChunkRawRecords(object):
         :param time_zero: Starting time of the first chunk
         """
         samples_per_record = strax.DEFAULT_RECORD_LENGTH
+        if len(instructions) == 0: # Empty
+            yield from np.array([], dtype=strax.raw_record_dtype(samples_per_record=samples_per_record))
+            self.rawdata.source_finished = True
+            return
         dt = self.config['sample_duration']
         buffer_length = len(self.record_buffer)
         rext = int(self.config['right_raw_extension'])
@@ -535,6 +539,24 @@ class RawRecordsFromFax1T(RawRecordsFromFaxNT):
     provides = ('raw_records', 'truth')
 
 
+@export
+class RawRecordsFromFaxOpticalNT(RawRecordsFromFaxNT):
+    
+    def _setup(self):
+        self.sim = ChunkRawRecords(self.config,
+                                   rawdata_generator=wfsim.RawDataOptical,
+                                   channels=self.channels,
+                                   timings=self.timings,)
+        self.sim.truth_buffer = np.zeros(10000, dtype=instruction_dtype + optical_extra_dtype
+                                                + truth_extra_dtype + [('fill', bool)])
+        self.sim_iter = self.sim(self.instructions)
+        
+        
+    def get_instructions(self):
+        assert self.config['fax_file'].endswith('.root'), 'You need to supply a root file for optical simulation!'
+        self.instructions, self.channels, self.timings = read_optical(self.config)
+        
+    
 @export
 @strax.takes_config(
     strax.Option('epix_config', track=False, default={},
@@ -596,6 +618,7 @@ class RawRecordsFromMcChain(SimulatorPlugin):
             self.instructions_epix = epix.run_epix.main(
                 epix.run_epix.setup(epix_config),
                 return_wfsim_instructions=True)
+
             self.g4id.append(self.instructions_epix['g4id'])
             log.debug("Epix produced %d instructions in tpc" % (len(self.instructions_epix)))
 
@@ -714,8 +737,11 @@ class RawRecordsFromMcChain(SimulatorPlugin):
                     log.debug('TPC instructions are already depleted')
                     result = dict([(data_type, np.zeros(0, self.dtype_for(data_type)))
                                    for data_type in self.provides if 'nv' not in data_type])
-                    self.sim.chunk_time = self.sim_nv.chunk_time
-                    self.sim.chunk_time_pre = self.sim_nv.chunk_time_pre
+
+                    num_of_results = sum([len(v) for _, v in result.items()])
+                    if num_of_results != 0:
+                        self.sim.chunk_time = self.sim_nv.chunk_time
+                        self.sim.chunk_time_pre = self.sim_nv.chunk_time_pre
                 else:
                     raise RuntimeError("Bug in getting source finished")
 
@@ -733,25 +759,50 @@ class RawRecordsFromMcChain(SimulatorPlugin):
                 else:
                     raise RuntimeError("Bug in getting source finished")
 
+        exist_tpc_result, exist_nveto_result = False, False
+        for data_type in self.provides:
+            if 'nv' in data_type:
+                if len(result_nv[data_type.strip('_nv')]) > 0:
+                    exist_nveto_result = True
+            else:
+                if len(result[data_type]) > 0:
+                    exist_tpc_result = True
         chunk = {}
         for data_type in self.provides:
             if 'nv' in data_type:
-                if 'nveto' in self.config['targets']:
-                  chunk[data_type] = self.chunk(start=self.sim_nv.chunk_time_pre,
+                if exist_nveto_result:
+                    chunk[data_type] = self.chunk(start=self.sim_nv.chunk_time_pre,
                                                 end=self.sim_nv.chunk_time,
                                                 data=result_nv[data_type.strip('_nv')],
                                                 data_type=data_type)
-                #If nv is not one of the targets just return an empty chunk
+                # If nv is not one of the targets just return an empty chunk
+                # If there is TPC event, set TPC time for the start and end
                 else:
-                  chunk[data_type] = self.chunk(start=self.sim.chunk_time_pre,
+                    dummy_dtype = wfsim.truth_extra_dtype if 'truth' in data_type else strax.raw_record_dtype()
+                    if exist_tpc_result:
+                        chunk[data_type] = self.chunk(start=self.sim.chunk_time_pre,
                                                 end=self.sim.chunk_time,
-                                                data=np.array([]),
+                                                data=np.array([], dtype=dummy_dtype),
                                                 data_type=data_type)
+                    else:
+                        chunk[data_type] = self.chunk(start=0, end=0, data=np.array([], dtype=dummy_dtype),
+                                                  data_type=data_type)
             else:
-                chunk[data_type] = self.chunk(start=self.sim.chunk_time_pre,
-                                              end=self.sim.chunk_time,
+                if exist_tpc_result:
+                    chunk[data_type] = self.chunk(start=self.sim.chunk_time_pre,
+                                                  end=self.sim.chunk_time,
                                               data=result[data_type],
                                               data_type=data_type)
+                else:
+                    dummy_dtype = wfsim.truth_extra_dtype if 'truth' in data_type else strax.raw_record_dtype()
+                    if exist_nveto_result:
+                        chunk[data_type] = self.chunk(start=self.sim_nv.chunk_time_pre,
+                                                      end=self.sim_nv.chunk_time,
+                                                  data=np.array([], dtype=dummy_dtype),
+                                                  data_type=data_type)
+                    else:
+                        chunk[data_type] = self.chunk(start=0, end=0, data=np.array([], dtype=dummy_dtype),
+                                                  data_type=data_type)
 
         self._sort_check([chunk[data_type].data for data_type in self.provides])
 
